@@ -123,7 +123,10 @@ NodeModel = T.TypeVar("NodeModel", bound="Node")
 ResolverType = T.TypeVar("ResolverType", bound="Resolver")
 ResolverTempType = T.TypeVar("ResolverTempType", bound="Resolver")
 
+OtherNodeType = T.TypeVar("OtherNodeType", bound="Node")
+
 GQL_D = T.Dict[str, T.Any]
+REL_D_INPUT = T.Dict[str, T.Optional[T.Union[OtherNodeType, T.List[OtherNodeType]]]]
 
 
 class Node(BaseModel):
@@ -252,7 +255,9 @@ class Node(BaseModel):
 
     """CRUDS"""
 
-    def base_set_remove_d(self) -> T.Tuple[GQL_D, GQL_D]:
+    def base_set_remove_d(
+        self, to_set: REL_D_INPUT = None, to_remove: REL_D_INPUT = None
+    ) -> T.Tuple[GQL_D, GQL_D]:
         set_d: GQL_D = {}
         remove_d: GQL_D = {}
 
@@ -269,22 +274,53 @@ class Node(BaseModel):
         for new_field_name in new_field_names:
             set_d[new_field_name] = curr_dict[new_field_name]
 
+        # now add relationship fields
+        rel_set_d = self.make_gql_d_from_rel_d_input(rel_d_input=to_set or {})
+        rel_remove_d = self.make_gql_d_from_rel_d_input(rel_d_input=to_remove or {})
+        set_d.update(rel_set_d)
+        remove_d.update(rel_remove_d)
+
         return set_d, remove_d
 
     async def update(self, given_resolver: ResolverType = None) -> bool:
-        """Will usually be overwritten assuming there are relationshops with this node"""
-        set_d, remove_d = self.base_set_remove_d()
+        """
+        Will usually be overwritten assuming there are relationshops
+        with this node (fill in to set and to remove with those relationship nodes)
+        """
         return await self._update(
-            set_d=set_d, remove_d=remove_d, given_resolver=given_resolver
+            to_set={}, to_remove={}, given_resolver=given_resolver
         )
 
+    @classmethod
+    async def _add(
+        cls: T.Type[NodeModel],
+        *,
+        input: BaseModel,
+        given_resolver: T.Optional[ResolverType] = None,
+        upsert: bool = False,
+        relationships_input: REL_D_INPUT = None,
+    ) -> NodeModel:
+        resolver = given_resolver or cls.Dgraph.resolver()
+        query_str = resolver.make_add_mutation_str()
+        input_d = jsonable_encoder(input)
+
+        rel_d = cls.make_gql_d_from_rel_d_input(rel_d_input=relationships_input or {})
+        input_d.update(rel_d)
+        variables = {"input": input_d, "upsert": upsert}
+        debug(query_str)
+        debug(variables)
+        j = await gql(query_str=query_str, variables=variables)
+        node_d = j["data"][f"add{cls.Dgraph.typename}"][cls.Dgraph.payload_node_name][0]
+        return resolver.parse_obj_nested(node_d)
+
     @staticmethod
-    def relationships_set_remove_d(
-        to_set: T.Dict[str, T.Optional[T.Union[NodeModel, T.List[NodeModel]]]],
-        to_remove: T.Dict[str, T.Optional[T.Union[NodeModel, T.List[NodeModel]]]],
-    ) -> T.Tuple[GQL_D, GQL_D]:
-        set_d: GQL_D = {}
-        for field_name, node_or_nodes in to_set.items():
+    def make_gql_d_from_rel_d_input(rel_d_input: REL_D_INPUT) -> GQL_D:
+        """
+        takes {taught_by: Teacher, is_friends_with: T.List[Student]} ->
+        {taught_by: {id: 0x1}, is_friends_with: [{id: 0x2, id: 0x3}] }}
+        """
+        gql_d_output: GQL_D = {}
+        for field_name, node_or_nodes in rel_d_input.items():
             if node_or_nodes is None:
                 continue
             if type(node_or_nodes) is list:
@@ -292,24 +328,31 @@ class Node(BaseModel):
             else:
                 node_or_nodes: NodeModel
                 val = {"id": node_or_nodes.id}
-            set_d[field_name] = val
-        remove_d: GQL_D = {}
-        for field_name, node_or_nodes in to_remove.items():
-            if node_or_nodes is None:
-                continue
-            if type(node_or_nodes) is list:
-                val = {"id": n.id for n in node_or_nodes}
-            else:
-                node_or_nodes: NodeModel
-                val = {"id": node_or_nodes.id}
-            remove_d[field_name] = val
-        return set_d, remove_d
+            gql_d_output[field_name] = val
+        return gql_d_output
 
     async def _update(
         self,
+        *,
+        given_resolver: T.Optional[ResolverType] = None,
+        to_set: T.Optional[REL_D_INPUT] = None,
+        to_remove: T.Optional[REL_D_INPUT] = None,
+        print_update_d: bool = False,
+    ) -> bool:
+        set_d, remove_d = self.base_set_remove_d(to_set=to_set, to_remove=to_remove)
+        return await self.update_from_set_remove_ds(
+            set_d=set_d,
+            remove_d=remove_d,
+            given_resolver=given_resolver,
+            print_update_d=print_update_d,
+        )
+
+    async def update_from_set_remove_ds(
+        self,
+        *,
+        given_resolver: T.Optional[ResolverType] = None,
         set_d: GQL_D,
         remove_d: GQL_D,
-        given_resolver: T.Optional[ResolverType] = None,
         print_update_d: bool = False,
     ) -> bool:
 
